@@ -276,7 +276,7 @@ TYPE specifies the archiver's symbol."
       (setq numbers (sort file-list '<))
       (elmo-living-messages numbers killed))))
 
-(defun elmo-archive-list-folder (spec)
+(defun elmo-archive-list-folder (spec &optional nohide)
   (elmo-archive-list-folder-subr spec))
 
 (defun elmo-archive-max-of-folder (spec)
@@ -460,7 +460,7 @@ TYPE specifies the archiver's symbol."
 ;;; Article file related functions
 ;;; read(extract) / append(move) / delete(delete) / query(list)
 
-(defun elmo-archive-read-msg (spec number outbuf)
+(defun elmo-archive-read-msg (spec number outbuf &optional msgdb unread)
   (save-excursion
     (let* ((type (nth 2 spec))
 	   (arc (elmo-archive-get-archive-name (nth 1 spec) type spec))
