@@ -49,7 +49,7 @@
       (setq numbers (sort flist '<))
       (elmo-living-messages numbers killed))))
 
-(defun elmo-internal-list-folder (spec)
+(defun elmo-internal-list-folder (spec &optional nohide)
   (elmo-internal-list-folder-subr spec))
 
 (defun elmo-internal-list-folder-by-location (spec location &optional msgdb)
@@ -193,7 +193,7 @@
   (let ((pair (assq number loc-alist)))
     (elmo-msgdb-global-mark-delete (cdr pair))))
 
-(defun elmo-internal-read-msg (spec number outbuf &optional msgdb)
+(defun elmo-internal-read-msg (spec number outbuf &optional msgdb unread)
   (save-excursion
     (let* ((loc-alist (if msgdb (elmo-msgdb-get-location msgdb)
 			(elmo-msgdb-location-load (elmo-msgdb-expand-path
