@@ -329,7 +329,7 @@ file name for maildir directories."
 	(progn (delete-file file)
 	       t))))
 
-(defun elmo-maildir-read-msg (spec number outbuf &optional msgdb)
+(defun elmo-maildir-read-msg (spec number outbuf &optional msgdb unread)
   (save-excursion
     (let* ((loc-alist (if msgdb (elmo-msgdb-get-location msgdb)
 			(elmo-msgdb-location-load (elmo-msgdb-expand-path
@@ -370,7 +370,7 @@ file name for maildir directories."
       (setq numbers (sort flist '<))
       (elmo-living-messages numbers killed))))
 
-(defun elmo-maildir-list-folder (spec)
+(defun elmo-maildir-list-folder (spec &optional nohide)
   (elmo-maildir-update-current spec)
   (elmo-maildir-list-folder-subr spec))
 
