@@ -40,12 +40,28 @@
 (eval-and-compile
   (autoload 'md5 "md5"))
 
-(defvar elmo-pop3-use-uidl t
-  "*If non-nil, use UIDL.")
-
 (defvar elmo-pop3-exists-exactly t)
+(defvar sasl-mechanism-alist)
 
-(luna-define-class elmo-pop3-session (elmo-network-session) ())
+(defvar elmo-pop3-total-size nil)
+
+;; For debugging.
+(defvar elmo-pop3-debug nil
+  "Non-nil forces POP3 folder as debug mode.
+Debug information is inserted in the buffer \"*POP3 DEBUG*\"")
+
+(defvar elmo-pop3-debug-inhibit-logging nil)
+
+;;; Debug
+(defsubst elmo-pop3-debug (message &rest args)
+  (if elmo-pop3-debug
+      (with-current-buffer (get-buffer-create "*POP3 DEBUG*")
+	(goto-char (point-max))
+	(if elmo-pop3-debug-inhibit-logging
+	    (insert "NO LOGGING\n")
+	  (insert (apply 'format message args) "\n")))))
+
+(luna-define-class elmo-pop3-session (elmo-network-session))
 
 ;; buffer-local
 (defvar elmo-pop3-read-point nil)
@@ -54,13 +70,15 @@
 (defvar elmo-pop3-size-hash nil) ; number -> size
 (defvar elmo-pop3-uidl-done nil)
 (defvar elmo-pop3-list-done nil)
+(defvar elmo-pop3-lock nil)
 
 (defvar elmo-pop3-local-variables '(elmo-pop3-read-point
 				    elmo-pop3-uidl-number-hash
 				    elmo-pop3-number-uidl-hash
 				    elmo-pop3-uidl-done
 				    elmo-pop3-size-hash
-				    elmo-pop3-list-done))
+				    elmo-pop3-list-done
+				    elmo-pop3-lock))
 
 (luna-define-method elmo-network-close-session ((session elmo-pop3-session))
   (when (elmo-network-session-process-internal session)
@@ -93,10 +111,12 @@
       (erase-buffer))
     (goto-char (point-min))
     (setq elmo-pop3-read-point (point))
+    (elmo-pop3-lock)
+    (elmo-pop3-debug "SEND: %s\n" command)
     (process-send-string process command)
     (process-send-string process "\r\n")))
 
-(defun elmo-pop3-read-response (process &optional not-command)
+(defun elmo-pop3-read-response (process &optional not-command keep-lock)
   (with-current-buffer (process-buffer process)
     (let ((case-fold-search nil)
 	  (response-string nil)
@@ -133,13 +153,19 @@
 		      (concat return-value "\n" response-string)
 		    response-string)))
 	  (setq elmo-pop3-read-point match-end)))
+      (unless keep-lock
+	(elmo-pop3-unlock))
       return-value)))
 
 (defun elmo-pop3-process-filter (process output)
   (save-excursion
     (set-buffer (process-buffer process))
     (goto-char (point-max))
-    (insert output)))
+    (insert output)
+    (elmo-pop3-debug "RECEIVED: %s\n" output)
+    (if elmo-pop3-total-size
+	(message "Retrieving...(%d/%d bytes)." 
+		 (buffer-size) elmo-pop3-total-size))))
 
 (defun elmo-pop3-auth-user (session)
   (let ((process (elmo-network-session-process-internal session)))
@@ -191,6 +217,7 @@
   (let ((process (elmo-network-session-process-internal session))
 	response mechanism)
     (with-current-buffer (process-buffer process)
+      (elmo-pop3-lock)
       (set-process-filter process 'elmo-pop3-process-filter)
       (setq elmo-pop3-read-point (point-min))
       ;; Skip garbage output from process before greeting.
@@ -220,9 +247,11 @@
   (with-current-buffer (process-buffer 
 			(elmo-network-session-process-internal session))
     (let* ((process (elmo-network-session-process-internal session))
+	   (elmo-pop3-debug-inhibit-logging t)
 	   (auth (elmo-network-session-auth-internal session))
 	   (auth (mapcar '(lambda (mechanism) (upcase (symbol-name mechanism)))
 			 (if (listp auth) auth (list auth))))
+	   sasl-mechanisms
 	   client name step response mechanism
 	   sasl-read-passphrase)
       (or (and (string= "USER" (car auth))
@@ -230,6 +259,8 @@
 	  (and (string= "APOP" (car auth))
 	       (elmo-pop3-auth-apop session))
 	  (progn
+	    (require 'sasl)
+	    (setq sasl-mechanisms (mapcar 'car sasl-mechanism-alist))
 	    (setq mechanism (sasl-find-mechanism auth))
 	    (unless mechanism
 	      (signal 'elmo-authenticate-error '(elmo-pop3-auth-no-mechanisms)))
@@ -254,23 +285,26 @@
 	     process
 	     (concat "AUTH " name
 		     (and (sasl-step-data step)
-			  (concat 
+			  (concat
 			   " "
 			   (elmo-base64-encode-string
 			    (sasl-step-data step) 'no-line-break))))) ;)
 	    (catch 'done
 	      (while t
 		(unless (setq response (elmo-pop3-read-response process t))
+		  ;; response is NO or BAD.
 		  (signal 'elmo-authenticate-error
 			  (list (intern
 				 (concat "elmo-pop3-auth-"
 					 (downcase name))))))
 		(if (string-match "^\+OK" response)
 		    (if (sasl-next-step client step)
+			;; Bogus server?
 			(signal 'elmo-authenticate-error
 				(list (intern
 				       (concat "elmo-pop3-auth-"
 					       (downcase name)))))
+		      ;; The authentication process is finished.
 		      (throw 'done nil)))
 		(sasl-step-set-data
 		 step
@@ -292,7 +326,7 @@
       (setq elmo-pop3-size-hash (elmo-make-hash 31))
       ;; To get obarray of uidl and size
       (elmo-pop3-send-command process "list")
-      (if (null (elmo-pop3-read-response process))
+      (if (null (elmo-pop3-read-response process nil 'keep-lock))
 	  (error "POP LIST command failed"))
       (if (null (setq response
 		      (elmo-pop3-read-contents
@@ -306,7 +340,7 @@
 	(setq elmo-pop3-number-uidl-hash (elmo-make-hash (* count 2)))
 	;; UIDL
 	(elmo-pop3-send-command process "uidl")
-	(unless (elmo-pop3-read-response process)
+	(unless (elmo-pop3-read-response process nil 'keep-lock)
 	  (error "POP UIDL failed"))
 	(unless (setq response (elmo-pop3-read-contents
 				(current-buffer) process))
@@ -314,8 +348,7 @@
 	(elmo-pop3-parse-uidl-response response)))))
 
 (defun elmo-pop3-read-contents (buffer process)
-  (save-excursion
-    (set-buffer buffer)
+  (with-current-buffer buffer
     (let ((case-fold-search nil)
 	  match-end)
       (goto-char elmo-pop3-read-point)
@@ -323,6 +356,7 @@
 	(accept-process-output process)
 	(goto-char elmo-pop3-read-point))
       (setq match-end (point))
+      (elmo-pop3-unlock)
       (elmo-delete-cr
        (buffer-substring elmo-pop3-read-point
 			 (- match-end 3))))))
@@ -425,7 +459,7 @@
 	    (sort list '<))
 	(error "POP3: Error in list")))))
 
-(defun elmo-pop3-list-folder (spec)
+(defun elmo-pop3-list-folder (spec &optional nohide)
   (let ((killed (and elmo-use-killed-list
 		     (elmo-msgdb-killed-list-load
 		      (elmo-msgdb-expand-path spec))))
@@ -471,6 +505,23 @@
       nil))
    (t
     nil)))
+
+(defun elmo-pop3-lock ()
+  "Lock pop3 process."
+  (setq elmo-pop3-lock t))
+
+(defun elmo-pop3-unlock ()
+  "Unlock pop3 process."
+  (setq elmo-pop3-lock nil))
+
+(defun elmo-pop3-locked-p (process)
+  "Return t if pop3 PROCESS is locked."
+  (with-current-buffer (process-buffer process)
+    (if elmo-pop3-lock
+	(progn
+	  (elmo-pop3-debug "POP3 is LOCKED!")
+	  t)
+      nil)))
      
 (defun elmo-pop3-retrieve-headers (buffer tobuffer process articles)
   (save-excursion
@@ -680,12 +731,13 @@
 	(accept-process-output process)
 	(goto-char start))
       (setq end (point))
+      (elmo-pop3-unlock)
       (with-current-buffer outbuf
 	(erase-buffer)
 	(insert-buffer-substring (process-buffer process) start (- end 3))
 	(elmo-delete-cr-get-content-type)))))
 
-(defun elmo-pop3-read-msg (spec number outbuf &optional msgdb)
+(defun elmo-pop3-read-msg (spec number outbuf &optional msgdb unread)
   (let* ((loc-alist (if elmo-pop3-use-uidl
 			(if msgdb
 			    (elmo-msgdb-get-location msgdb)
@@ -693,18 +745,34 @@
 			   (elmo-msgdb-expand-path spec)))))
 	 (process (elmo-network-session-process-internal
 		   (elmo-pop3-get-session spec)))
-	 response errmsg msg)
+	 size response errmsg msg)
     (with-current-buffer (process-buffer process)
       (if loc-alist
 	  (setq number (elmo-pop3-uidl-to-number
 			(cdr (assq number loc-alist)))))
+      (setq size (string-to-number
+		  (elmo-pop3-number-to-size number)))
       (when number
 	(elmo-pop3-send-command process
 				(format "retr %s" number))
-	(when (null (setq response (elmo-pop3-read-response
-				    process t)))
-	  (error "Fetching message failed"))
-	(setq response (elmo-pop3-read-body process outbuf))
+	(setq elmo-pop3-total-size size)
+	(unless elmo-inhibit-display-retrieval-progress
+	  (setq elmo-pop3-total-size size)
+	  (elmo-display-progress
+	   'elmo-pop3-display-retrieval-progress
+	   (format "Retrieving (0/%d bytes)..." elmo-pop3-total-size)
+	   0))
+	(unwind-protect
+	    (progn
+	      (when (null (setq response (elmo-pop3-read-response
+					  process t 'keep-lock)))
+		(error "Fetching message failed"))
+	      (setq response (elmo-pop3-read-body process outbuf)))
+	  (setq elmo-pop3-total-size nil))
+	(unless elmo-inhibit-display-retrieval-progress
+	  (elmo-display-progress
+	   'elmo-display-retrieval-progress "" 100)  ; remove progress bar.
+	  (message "Retrieving...done."))
 	(set-buffer outbuf)
 	(goto-char (point-min))
 	(while (re-search-forward "^\\." nil t)
@@ -780,8 +848,10 @@
 (defun elmo-pop3-commit (spec)
   (if (elmo-pop3-plugged-p spec)
       (let ((session (elmo-pop3-get-session spec 'if-exists)))
-	(and session
-	     (elmo-network-close-session session)))))
+	(when (and session
+		   (not (elmo-pop3-locked-p
+			 (elmo-network-session-process-internal session))))
+	  (elmo-network-close-session session)))))
        
 
 (require 'product)
