@@ -508,7 +508,7 @@ File content is encoded with MIME-CHARSET."
 				   "Since" "Before" "ToCc"
 				   "!From" "!Subject" "!To" "!Cc" "!Body"
 				   "!Since" "!Before" "!ToCc")
-				 elmo-msgdb-extra-fields))))
+				 elmo-msgdb-extra-fields)) nil t))
 	 value)
     (setq field (if (string= field "")
 		    (setq field default)
@@ -682,6 +682,20 @@ Return value is a cons cell of (STRUCTURE . REST)"
 					  (cdr tmp)))))))
   lst)
 
+(defun elmo-list-insert (list element after)
+  "Insert an ELEMENT to the LIST, just after AFTER."
+  (let ((li list)
+	(curn 0)
+	p pn)
+    (while li
+      (if (eq (car li) after)
+	  (setq p li pn curn li nil)
+	(incf curn))
+      (setq li (cdr li)))
+    (if pn
+	(setcdr (nthcdr pn list) (cons element (cdr p)))
+      (nconc list (list element)))))
+
 (defun elmo-string-partial-p (string)
   (and (stringp string) (string-match "message/partial" string)))
 
@@ -1355,13 +1369,48 @@ Otherwise treat \\ in NEWTEXT string as special:
 	(elmo-buffer-field-condition-match
 	 (nth 2 condition) number number-list)))))
 
-(defsubst elmo-file-field-condition-match (file condition number number-list)
-  (elmo-set-work-buf
-   (as-binary-input-file (insert-file-contents file))
-   (elmo-set-buffer-multibyte default-enable-multibyte-characters)
-   ;; Should consider charset?
-   (decode-mime-charset-region (point-min)(point-max) elmo-mime-charset)
-   (elmo-buffer-field-condition-match condition number number-list)))
+(defsubst elmo-file-field-primitive-condition-match (file
+						     condition
+						     number
+						     number-list)
+  (let (result)
+    (goto-char (point-min))
+    (cond
+     ((string= (elmo-filter-key condition) "last")
+      (setq result (<= (length (memq number number-list))
+		       (string-to-int (elmo-filter-value condition)))))
+     ((string= (elmo-filter-key condition) "first")
+      (setq result (< (- (length number-list)
+			 (length (memq number number-list)))
+		      (string-to-int (elmo-filter-value condition)))))
+     (t
+      (elmo-set-work-buf
+       (as-binary-input-file (insert-file-contents file))
+       (elmo-set-buffer-multibyte default-enable-multibyte-characters)
+       ;; Should consider charset?
+       (decode-mime-charset-region (point-min)(point-max) elmo-mime-charset)
+       (setq result
+	     (elmo-buffer-field-primitive-condition-match
+	      condition number number-list)))))
+    (if (eq (elmo-filter-type condition) 'unmatch)
+	(setq result (not result)))
+    result))
+
+(defun elmo-file-field-condition-match (file condition number number-list)
+  (cond
+   ((vectorp condition)
+    (elmo-file-field-primitive-condition-match
+     file condition number number-list))
+   ((eq (car condition) 'and)
+    (and (elmo-file-field-condition-match
+	  file (nth 1 condition) number number-list)
+	 (elmo-file-field-condition-match
+	  file (nth 2 condition) number number-list)))
+   ((eq (car condition) 'or)
+    (or (elmo-file-field-condition-match
+	 file (nth 1 condition) number number-list)
+	(elmo-file-field-condition-match
+	 file (nth 2 condition) number number-list)))))
 
 (defmacro elmo-get-hash-val (string hashtable)
   (let ((sym (list 'intern-soft string hashtable)))
@@ -1566,10 +1615,10 @@ the value of `foo'."
 		(list 'error-message doc
 		      'error-conditions (cons error conds))))))
 
-(cond ((fboundp 'lprogress-display)
-       (defalias 'elmo-display-progress 'lprogress-display))
-      ((fboundp 'progress-feedback-with-label)
+(cond ((fboundp 'progress-feedback-with-label)
        (defalias 'elmo-display-progress 'progress-feedback-with-label))
+      ((fboundp 'lprogress-display)
+       (defalias 'elmo-display-progress 'lprogress-display))
       (t
        (defun elmo-display-progress (label format &optional value &rest args)
 	 "Print a progress message."
