@@ -71,6 +71,8 @@ Any symbol value of `elmo-network-stream-type-alist'.")
 (defvar elmo-pop3-stream-type-alist nil
   "*Stream bindings for POP3.
 This is taken precedence over `elmo-network-stream-type-alist'.")
+(defvar elmo-pop3-use-uidl t
+  "*If non-nil, use UIDL.")
 
 ;; NNTP
 (defvar elmo-default-nntp-server  "localhost"
@@ -174,14 +176,12 @@ For disconnected operations.")
 (defvar elmo-imap4-use-select-to-update-status nil
   "*Some imapd have to send select command to update status.
 (ex. UW imapd 4.5-BETA?).  For these imapd, you must set this variable t.")
-
 (defvar elmo-imap4-use-modified-utf7 nil
   "*Use mofidied UTF-7 (rfc2060) encoding for IMAP4 folder name.")
 
 (defvar elmo-auto-change-plugged 600
   "*Time to expire change plugged state automatically, as the number of seconds.
 Don't change plugged state automatically if nil.")
-
 (defvar elmo-plugged-condition 'one
   "*The condition for `elmo-plugged' becomes on.
 If `all', when all ports are on.  If `one', when even one port is on.
@@ -360,6 +360,15 @@ set as non-nil.")
 (defvar elmo-display-progress-threshold 20
   "*Displaying progress gauge if number of messages are more than this value.")
 
+(defvar elmo-inhibit-read-cache nil
+  "*Global switch to inhibit reading cache.")
+
+(defvar elmo-inhibit-display-retrieval-progress nil
+  "Global switch to inhibit display progress of each message's retrieval.")
+
+(defvar elmo-display-retrieval-progress-threshold 30000
+  "*Don't display progress if the message size is smaller than this value.")
+
 (require 'product)
 (product-provide (provide 'elmo-vars) (require 'elmo-version))
 
