@@ -174,13 +174,13 @@
 (defun wl-draft-insert-x-face-field ()
   "Insert X-Face header."
   (interactive)
-  (if (not (file-exists-p wl-x-face-file))
-      (error "File %s does not exist" wl-x-face-file)
-    (beginning-of-buffer)
-    (search-forward mail-header-separator nil t)
-    (beginning-of-line)
-    (wl-draft-insert-x-face-field-here)
-    (run-hooks 'wl-draft-insert-x-face-field-hook))) ; highlight it if you want.
+  (unless (file-exists-p wl-x-face-file)
+    (error "File %s does not exist" wl-x-face-file))
+  (beginning-of-buffer)
+  (search-forward mail-header-separator nil t)
+  (beginning-of-line)
+  (wl-draft-insert-x-face-field-here)
+  (run-hooks 'wl-draft-insert-x-face-field-hook)) ; highlight it if you want.
 
 (defun wl-draft-insert-x-face-field-here ()
   "Insert X-Face field at point."
@@ -453,15 +453,11 @@ Reply to author if WITH-ARG is non-nil."
 (defun wl-draft-confirm ()
   "Confirm send message."
   (interactive)
-  (y-or-n-p (format "Send current draft as %s? "
-		    (cond ((and (wl-message-mail-p) (wl-message-news-p))
-			   "Mail and News")
-			  ((wl-message-mail-p) "Mail")
-			  ((wl-message-news-p) "News")))))
-
-(defun wl-message-news-p ()
-  "If exist valid Newsgroups field, return non-nil."
-  (std11-field-body "Newsgroups"))
+  (y-or-n-p
+   (cond ((and (wl-message-mail-p) (wl-message-news-p))
+	  "Send current draft as Mail and News? ")
+	 ((wl-message-mail-p) "Send current draft as Mail? ")
+	 ((wl-message-news-p) "Send current draft as News? "))))
 
 (defun wl-message-field-exists-p (field)
   "If FIELD exist and FIELD value is not empty, return non-nil."
@@ -469,6 +465,10 @@ Reply to author if WITH-ARG is non-nil."
     (and value
 	 (not (string= value "")))))
 
+(defun wl-message-news-p ()
+  "If exist valid Newsgroups field, return non-nil."
+  (std11-field-body "Newsgroups"))
+
 (defun wl-message-mail-p ()
   "If exist To, Cc or Bcc field, return non-nil."
   (or (wl-message-field-exists-p "To")
@@ -491,9 +491,8 @@ Reply to author if WITH-ARG is non-nil."
   (let ((cur-buf (current-buffer))
 	(tmp-buf (get-buffer-create " *wl-draft-edit-string*"))
 	to subject in-reply-to cc references newsgroups mail-followup-to
-	content-type content-transfer-encoding
-	body-beg buffer-read-only
-	)
+	content-type content-transfer-encoding from
+	body-beg buffer-read-only)
     (set-buffer tmp-buf)
     (erase-buffer)
     (insert string)
@@ -509,6 +508,12 @@ Reply to author if WITH-ARG is non-nil."
 			(decode-mime-charset-string
 			 subject
 			 wl-mime-charset))))
+    (setq from (std11-field-body "From")
+	  from (and from
+		    (eword-decode-string
+		     (decode-mime-charset-string
+		      from
+		      wl-mime-charset))))    
     (setq in-reply-to (std11-field-body "In-Reply-To"))
     (setq cc (std11-field-body "Cc"))
     (setq cc (and cc
@@ -530,8 +535,10 @@ Reply to author if WITH-ARG is non-nil."
 		   mail-followup-to
 		   content-type content-transfer-encoding
 		   (buffer-substring (point) (point-max))
-		   'edit-again
-		   ))
+		   'edit-again nil
+		   (if (member (nth 1 (std11-extract-address-components from))
+			       wl-user-mail-address-list)
+		       from)))
       (and to (mail-position-on-field "To"))
       (delete-other-windows)
       (kill-buffer tmp-buf)))
@@ -543,10 +550,9 @@ Reply to author if WITH-ARG is non-nil."
   (let ((mail-reply-buffer (wl-message-get-original-buffer))
 	mail-citation-hook mail-yank-hooks
 	wl-draft-add-references wl-draft-cite-func)
-    (if (eq 0
-	    (save-excursion
-	      (set-buffer mail-reply-buffer)
-	      (buffer-size)))
+    (if (zerop
+	 (with-current-buffer mail-reply-buffer
+	   (buffer-size)))
 	(error "No current message")
       (wl-draft-yank-from-mail-reply-buffer nil
 					    wl-ignored-forwarded-headers))))
@@ -672,7 +678,7 @@ Reply to author if WITH-ARG is non-nil."
 	    (let ((msg (and wl-draft-buffer-file-name
 			    (string-match "[0-9]+$" wl-draft-buffer-file-name)
 			    (string-to-int
-			     (elmo-match-string 0 wl-draft-buffer-file-name)))))
+			     (match-string 0 wl-draft-buffer-file-name)))))
 	      (wl-draft-config-info-operation msg 'delete))))
       (set-buffer-modified-p nil)		; force kill
       (kill-buffer editing-buffer))))
@@ -691,11 +697,11 @@ Reply to author if WITH-ARG is non-nil."
     (message "")))
 
 (defun wl-draft-fcc ()
-  "Add a new FCC field, with file name completion."
+  "Add a new Fcc field, with file name completion."
   (interactive)
-  (or (mail-position-on-field "fcc" t)  ;Put new field after exiting FCC.
+  (or (mail-position-on-field "fcc" t)  ;Put new field after exiting Fcc.
       (mail-position-on-field "to"))
-  (insert "\nFCC: "))
+  (insert "\nFcc: "))
 
 ;; function for wl-sent-message-via
 
@@ -798,20 +804,11 @@ to find out how to use this."
 
 (defun wl-draft-parse-msg-id-list-string (string)
   "Get msg-id list from STRING."
-  (let ((parsed (std11-parse-msg-ids-string string))
-	tokens msg-id msg-id-list)
-    (while parsed
-      (setq msg-id nil)
-      (when (eq (car (car parsed)) 'msg-id)
-	(setq tokens (cdr (car parsed)))
-	(while tokens
-	  (if (or (eq (car (car tokens)) 'atom)
-		  (eq (car (car tokens)) 'specials))
-	      (setq msg-id (concat msg-id (cdr (car tokens)))))
-	  (setq tokens (cdr tokens))))
-      (if msg-id (setq msg-id-list (cons (concat "<" msg-id ">")
-					 msg-id-list)))
-      (setq parsed (cdr parsed)))
+  (let (msg-id-list)
+    (dolist (parsed-id (std11-parse-msg-ids-string string))
+      (when (eq (car parsed-id) 'msg-id)
+	(setq msg-id-list (cons (std11-msg-id-string parsed-id)
+				msg-id-list))))
     (nreverse msg-id-list)))
 
 (defun wl-draft-parse-mailbox-list (field &optional remove-group-list)
@@ -881,27 +878,6 @@ non-nil."
 	  (setq fields (cdr fields)))
 	recipients))))
 
-(static-if (fboundp 'smtp-send-buffer)
-(defun wl-draft-smtp-send-buffer (sender recipients buffer id)
-  (wl-smtp-extension-bind
-   (condition-case err
-       (smtp-send-buffer sender recipients (current-buffer))
-     (error
-      (wl-draft-write-sendlog 'failed 'smtp smtp-server
-			      recipients id)
-      (signal (car err) (cdr err))))))
-(defun wl-draft-smtp-send-buffer (sender recipients buffer id)
-  (as-binary-process
-   (wl-smtp-extension-bind
-    (let ((err (smtp-via-smtp sender recipients
-			      (current-buffer))))
-      (when (not (eq err t))
-	(wl-draft-write-sendlog 'failed 'smtp smtp-server
-				recipients id)
-	(error "Sending failed; SMTP protocol error:%s" err))))))
-;; end of static-if
-)
-
 ;;
 ;; from Semi-gnus
 ;;
@@ -960,12 +936,18 @@ non-nil."
 	    (wl-draft-delete-field "bcc" delimline)
 	    (wl-draft-delete-field "resent-bcc" delimline)
 	    (let (process-connection-type)
-	      (when recipients
-		(wl-draft-smtp-send-buffer sender recipients (current-buffer)
-					   id)
-		(wl-draft-set-sent-message 'mail 'sent)
-		(wl-draft-write-sendlog
-		 'ok 'smtp smtp-server recipients id))))
+	      (as-binary-process
+	       (when recipients
+		 (wl-smtp-extension-bind
+		  (condition-case err
+		      (smtp-send-buffer sender recipients (current-buffer))
+		    (error
+		     (wl-draft-write-sendlog 'failed 'smtp smtp-server
+					     recipients id)
+		     (signal (car err) (cdr err)))))
+		 (wl-draft-set-sent-message 'mail 'sent)
+		 (wl-draft-write-sendlog
+		  'ok 'smtp smtp-server recipients id)))))
 	(if (bufferp errbuf)
 	    (kill-buffer errbuf))))))
 
@@ -1057,11 +1039,11 @@ If FORCE-MSGID, ignore 'wl-insert-message-id'."
 	(if (and sent-via wl-draft-fcc-list)
 	    (progn
 	      (wl-draft-do-fcc (wl-draft-get-header-delimiter) wl-draft-fcc-list)
-	      (setq wl-draft-fcc-list nil))
-	  (if wl-draft-use-cache
-	      (let ((id (std11-field-body "Message-ID"))
-		    (elmo-enable-disconnected-operation t))
-		(elmo-cache-save id nil nil nil))))
+	      (setq wl-draft-fcc-list nil)))
+	(if wl-draft-use-cache
+	    (let ((id (std11-field-body "Message-ID"))
+		  (elmo-enable-disconnected-operation t))
+	      (elmo-cache-save id nil nil nil)))
 	;; If one unplugged, append queue.
 	(when (and unplugged-via
 		   wl-sent-message-modified)
@@ -1157,7 +1139,7 @@ If optional argument is non-nil, current draft buffer is killed"
   (wl-draft-config-info-operation
    (and (string-match "[0-9]+$" wl-draft-buffer-file-name)
 	(string-to-int
-	 (elmo-match-string 0 wl-draft-buffer-file-name)))
+	 (match-string 0 wl-draft-buffer-file-name)))
    'save))
 
 (defun wl-draft-mimic-kill-buffer ()
@@ -1225,7 +1207,7 @@ If optional argument is non-nil, current draft buffer is killed"
     (or (markerp header-end) (error "header-end must be a marker"))
     (save-excursion
       (goto-char (point-min))
-      (while (re-search-forward "^FCC:[ \t]*" header-end t)
+      (while (re-search-forward "^Fcc:[ \t]*" header-end t)
 	(setq fcc-list
 	      (cons (buffer-substring-no-properties
 		     (point)
@@ -1295,15 +1277,11 @@ If optional argument is non-nil, current draft buffer is killed"
 		nil
 	      (if (re-search-forward ":" pos t) nil t)))))))
 
-(defun wl-draft-random-alphabet ()
-  (let ((alphabet '(?A ?B ?C ?D ?E ?F ?G ?H ?I ?J ?K ?L ?M ?N ?O ?P ?Q ?R ?S ?T ?U ?V ?W ?X ?Y ?Z)))
-    (nth (abs (% (random) 26)) alphabet)))
-
 ;;;###autoload
 (defun wl-draft (&optional to subject in-reply-to cc references newsgroups
 			   mail-followup-to
 			   content-type content-transfer-encoding
-			   body edit-again summary-buf)
+			   body edit-again summary-buf from)
   "Write and send mail/news message with Wanderlust."
   (interactive)
   (unless (featurep 'wl)
@@ -1336,13 +1314,13 @@ If optional argument is non-nil, current draft buffer is killed"
     (if (or (eq wl-draft-reply-buffer-style 'full)
 	    (eq this-command 'wl-draft)
 	    (eq this-command 'wl-summary-write)
-	    (eq this-command 'wl-summary-write-current-newsgroup))
+	    (eq this-command 'wl-summary-write-current-folder))
 	(delete-other-windows))
     (auto-save-mode -1)
     (wl-draft-mode)
     (setq wl-sent-message-via nil)
-    (if (stringp wl-from)
-	(insert "From: " wl-from "\n"))
+    (if (stringp (or from wl-from))
+	(insert "From: " (or from wl-from) "\n"))
     (and (or (interactive-p)
 	     (eq this-command 'wl-summary-write)
 	     to)
@@ -1362,10 +1340,9 @@ If optional argument is non-nil, current draft buffer is killed"
     (setq wl-draft-buffer-file-name file-name)
     (if mail-default-reply-to
 	(insert "Reply-To: " mail-default-reply-to "\n"))
-    (if (or wl-bcc mail-self-blind)
-	(insert "Bcc: " (or wl-bcc (user-login-name)) "\n"))
-    (if wl-fcc
-	(insert "FCC: " (if (functionp wl-fcc) (funcall wl-fcc) wl-fcc) "\n"))
+    (wl-draft-insert-ccs "Bcc: " (or wl-bcc
+			       (and mail-self-blind (user-login-name))))
+    (wl-draft-insert-ccs "Fcc: " wl-fcc)
     (if wl-organization
 	(insert "Organization: " wl-organization "\n"))
     (and wl-auto-insert-x-face
@@ -1381,7 +1358,7 @@ If optional argument is non-nil, current draft buffer is killed"
 	  (when content-type
 	    (insert "Content-type: " content-type "\n"))
 	  (when content-transfer-encoding
-	    (insert "Content-Transfer-encoding: " content-transfer-encoding "\n"))
+	    (insert "Content-Transfer-Encoding: " content-transfer-encoding "\n"))
 	  (if (or content-type content-transfer-encoding)
 	      (insert "\n"))
 	  (and body (insert body))
@@ -1427,6 +1404,21 @@ If optional argument is non-nil, current draft buffer is killed"
 						  wl-summary-buffer-name)))
     buf-name))
 
+(defsubst wl-draft-insert-ccs (str cc)
+  (let ((field
+	 (if (functionp cc)
+	     (funcall cc)
+	   cc)))
+    (if (and field
+	     (null (and wl-draft-delete-myself-from-bcc-fcc
+			(elmo-list-member
+			 (mapcar 'wl-address-header-extract-address
+				 (append
+				  (wl-parse-addresses (std11-field-body "To"))
+				  (wl-parse-addresses (std11-field-body "Cc"))))
+			 (mapcar 'downcase wl-subscribed-mailing-list)))))
+	(insert str field "\n"))))
+
 (defun wl-draft-elmo-nntp-send ()
   (let ((elmo-nntp-post-pre-hook wl-news-send-pre-hook)
 	(elmo-default-nntp-user
