@@ -840,7 +840,7 @@ Optional argument ARG is repeart count."
 					     (elmo-msgdb-expand-path entity))
 					    entity)))
 					 (cdr nums))
-				   (current-buffer)))
+				   (get-buffer wl-folder-buffer-name)))
     (setq wl-folder-info-alist-modified t)
     (sit-for 0)
     (list (if wl-folder-notify-deleted
@@ -981,13 +981,13 @@ If current line is group folder, check all sub entries."
       (setq unread (or (cadr nums) 0))
       (if (or (not unread-only)
 	      (or (< 0 new) (< 0 unread)))
-	  (save-window-excursion
-	    (save-excursion
-	      (let ((wl-summary-buffer-name (concat
-					     wl-summary-buffer-name
-					     (symbol-name this-command)))
-		    (wl-message-buf-name (concat wl-message-buf-name
-						 (symbol-name this-command))))
+	  (let ((wl-summary-buffer-name (concat
+					 wl-summary-buffer-name
+					 (symbol-name this-command)))
+		(wl-message-buf-name (concat wl-message-buf-name
+					     (symbol-name this-command))))
+	    (save-window-excursion
+	      (save-excursion
 		(wl-summary-goto-folder-subr entity
 					     (wl-summary-get-sync-range entity)
 					     nil nil nil t)
@@ -1028,18 +1028,18 @@ If current line is group folder, check all subfolders."
       (setq new (or (car nums) 0))
       (setq unread (or (cadr nums) 0))
       (if (or (< 0 new) (< 0 unread))
-	(save-window-excursion
-	  (save-excursion
-	    (let ((wl-summary-buffer-name (concat
-					     wl-summary-buffer-name
-					     (symbol-name this-command)))
-		    (wl-message-buf-name (concat wl-message-buf-name
-						 (symbol-name this-command))))
-	      (wl-summary-goto-folder-subr entity
+	  (let ((wl-summary-buffer-name (concat
+					 wl-summary-buffer-name
+					 (symbol-name this-command)))
+		(wl-message-buf-name (concat wl-message-buf-name
+					     (symbol-name this-command))))
+	    (save-window-excursion
+	      (save-excursion
+		(wl-summary-goto-folder-subr entity
 					   (wl-summary-get-sync-range entity)
 					   nil)
-	      (wl-summary-mark-as-read-all)
-	      (wl-summary-exit))))
+		(wl-summary-mark-as-read-all)
+		(wl-summary-exit))))
 	(sit-for 0))))))
 
 (defun wl-folder-mark-as-read-all-current-entity ()
@@ -1286,19 +1286,21 @@ If current line is group folder, all subfolders are marked."
 
 (defun wl-folder-select-buffer (buffer)
   (let ((gbw (get-buffer-window buffer))
-	ret-val)
+	exists)
     (if gbw
 	(progn (select-window gbw)
-	       (setq ret-val t))
-      (condition-case ()
-	  (unwind-protect
-	      (split-window-horizontally wl-folder-window-width)
-	    (other-window 1))
-	(error nil)))
+	       (setq exists t))
+      (unless wl-summary-use-frame
+	(condition-case ()
+	    (unwind-protect
+		(split-window-horizontally wl-folder-window-width)
+	      (other-window 1))
+	  (error nil))))
     (set-buffer buffer)
-    (switch-to-buffer buffer)
-    ret-val
-    ))
+    (if wl-summary-use-frame
+	(switch-to-buffer-other-frame buffer)
+      (switch-to-buffer buffer))
+    exists))
 
 (defun wl-folder-toggle-disp-summary (&optional arg folder)
   (interactive)
@@ -1432,11 +1434,24 @@ Entering Folder mode calls the value of `wl-folder-mode-hook'."
 
 (defun wl-folder (&optional arg)
   (interactive "P")
-  (let (initialize)
-;;; (delete-other-windows)
-    (if (get-buffer wl-folder-buffer-name)
-	(switch-to-buffer  wl-folder-buffer-name)
-      (switch-to-buffer (get-buffer-create wl-folder-buffer-name))
+  (let (initialize folder-buf)
+    (if (setq folder-buf (get-buffer wl-folder-buffer-name))
+	(if wl-folder-use-frame
+	    (let (select-frame)
+	      (save-selected-window
+		(dolist (frame (visible-frame-list))
+		  (select-frame frame)
+		  (if (get-buffer-window folder-buf)
+		      (setq select-frame frame))))
+	      (if select-frame
+		  (select-frame select-frame)
+		(switch-to-buffer folder-buf)))
+	  (switch-to-buffer folder-buf))
+      (if wl-folder-use-frame
+	  (switch-to-buffer-other-frame
+	   (get-buffer-create wl-folder-buffer-name))
+	(switch-to-buffer (get-buffer-create wl-folder-buffer-name)))
+      (switch-to-buffer (get-buffer wl-folder-buffer-name))
       (wl-folder-mode)
       (wl-folder-init)
       (set-buffer wl-folder-buffer-name)
@@ -2064,7 +2079,13 @@ Use `wl-subscribed-mailing-list'."
   (when (memq (elmo-folder-get-type folder)
 	      '(localdir imap4 maildir))
     (let (key mladdress)
-      (when (string-match "[^\\./]+$" folder)
+      (setq folder			; make folder name simple
+	    (if (eq 'imap4 (elmo-folder-get-type folder))
+		(elmo-imap4-spec-mailbox (elmo-imap4-get-spec folder))
+	      (substring folder 1)))
+      (if (string-match "@" folder)
+	  (setq folder (substring folder 0 (match-beginning 0))))
+      (when (string-match "[^\\./]+$" folder) ; last hierarchy
 	(setq key (regexp-quote
 		   (concat (substring folder (match-beginning 0)) "@")))
 	(setq mladdress
@@ -2618,13 +2639,13 @@ Use `wl-subscribed-mailing-list'."
       (setq count (+ count (wl-folder-count-incorporates entity)))
       (if (or (null (car nums)) ; unknown
 	      (< 0 count))
-	  (save-window-excursion
-	    (save-excursion
-	      (let ((wl-summary-buffer-name (concat
-					     wl-summary-buffer-name
-					     (symbol-name this-command)))
-		    (wl-message-buf-name (concat wl-message-buf-name
-						 (symbol-name this-command))))
+	  (let ((wl-summary-buffer-name (concat
+					 wl-summary-buffer-name
+					 (symbol-name this-command)))
+		(wl-message-buf-name (concat wl-message-buf-name
+					     (symbol-name this-command))))
+	    (save-window-excursion
+	      (save-excursion
 		(wl-summary-goto-folder-subr entity
 					     (wl-summary-get-sync-range entity)
 					     nil)
@@ -2675,13 +2696,13 @@ If current line is group folder, all subfolders are prefetched."
 	  wl-summary-highlight wl-auto-select-first new)
       (setq new (or (car nums) 0))
       (if (< 0 new)
-	  (save-window-excursion
-	    (save-excursion
-	      (let ((wl-summary-buffer-name (concat
-					     wl-summary-buffer-name
-					     (symbol-name this-command)))
-		    (wl-message-buf-name (concat wl-message-buf-name
-						 (symbol-name this-command))))
+	  (let ((wl-summary-buffer-name (concat
+					 wl-summary-buffer-name
+					 (symbol-name this-command)))
+		(wl-message-buf-name (concat wl-message-buf-name
+					     (symbol-name this-command))))
+	    (save-window-excursion
+	      (save-excursion
 		(wl-summary-goto-folder-subr entity 'no-sync nil)
 		(wl-summary-drop-unsync)
 		(wl-summary-exit)))))))))
@@ -2710,10 +2731,12 @@ If optional arg exists, don't check any folders."
 	(message "All unsync messages in %s are dropped!" entity-name)))))
 
 (defun wl-folder-write-current-folder ()
-  ""
+  "Write message to current folder's newsgroup or mailing-list.
+Call `wl-summary-write-current-folder' with current folder name."
   (interactive)
   (unless (wl-folder-buffer-group-p)
-    (wl-summary-write-current-folder (wl-folder-entity-name))))
+    (wl-summary-write-current-folder
+     (wl-folder-get-realname (wl-folder-entity-name)))))
 
 (defun wl-folder-mimic-kill-buffer ()
   "Kill the current (Folder) buffer with query."
