@@ -374,13 +374,12 @@ Set `wl-score-cache' nil."
       (let ((mark (car (wl-score-get 'mark alist)))
 	    (expunge (car (wl-score-get 'expunge alist)))
 	    (mark-and-expunge (car (wl-score-get 'mark-and-expunge alist)))
-	    (temp (car (wl-score-get 'temp alist))) ; obsolate
 	    (target (car (wl-score-get 'target alist)))
 	    (important (car (wl-score-get 'important alist))))
 	(setq wl-summary-important-above
 	      (or important wl-summary-important-above))
 	(setq wl-summary-target-above
-	      (or target temp wl-summary-target-above))
+	      (or target wl-summary-target-above))
 	(setq wl-summary-mark-below
 	      (or mark mark-and-expunge wl-summary-mark-below))
 	(setq wl-summary-expunge-below
