@@ -1,7 +1,7 @@
-;;; wl-vars.el -- Variable definitions for Wanderlust.
+;;; wl-vars.el -- Variable definitions for Wanderlust. -*-coding:euc-japan;-*-
 
-;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
-;; Copyright (C) 1998,1999,2000 Masahiro MURATA <muse@ba2.so-net.ne.jp>
+;; Copyright (C) 1998,1999,2000,2001 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 1998,1999,2000,2001 Masahiro MURATA <muse@ba2.so-net.ne.jp>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;;	Masahiro MURATA <muse@ba2.so-net.ne.jp>
@@ -41,7 +41,12 @@
 (defgroup wl nil
   "Wanderlust, a news and mail reading software."
   :tag "Wanderlust"
-  :link '(custom-manual "(wl-ja)Top")
+  :link (` (custom-manual
+	    (, (if (and (boundp 'current-language-environment)
+			(string-equal "Japanese"
+				      (symbol-value 'current-language-environment)))
+		   "(wl-ja)Top"
+		 "(wl)Top"))))
   :group 'news
   :group 'mail)
 
@@ -178,7 +183,7 @@ If you don't have multiple e-mail addresses, you don't have to set this."
   :type 'function
   :group 'wl-summary)
 
-(defcustom wl-summary-search-parent-by-subject-regexp "^[ \t]*\\(\\[[^:]+[,: ][0-9]+\\]\\)?[ \t]*re[\\^[: ]"
+(defcustom wl-summary-search-parent-by-subject-regexp "^[ \t]*\\(\\[[^:]+[,: ][0-9]+\\]\\)?[ \t]*re[\\^[:> ]"
   "*If message does not have in-reply-to field nor references field and
  subject matches this regexp, search parent message by subject matching."
   :type 'string
@@ -462,10 +467,16 @@ reasons of system internal to accord facilities for the Emacs variants.")
   "A hook called when header is highlighted.")
 (defvar wl-highlight-message-hook nil
   "A hook called when message is highlighted.")
+(defvar wl-save-hook nil
+  "A hook called when save summary and folder status.")
 (defvar wl-exit-hook nil
   "A hook called when exit wanderlust.")
 (defvar wl-folder-suspend-hook nil
   "A hook called when suspend wanderlust.")
+(defvar wl-biff-notify-hook nil
+  "A hook called when a biff-notification is invoked.")
+(defvar wl-biff-unnotify-hook nil
+  "A hook called when a biff-notification is removed.")
 (defvar wl-auto-check-folder-pre-hook nil
   "A hook called before auto check folders.")
 (defvar wl-auto-check-folder-hook nil
@@ -605,6 +616,12 @@ Default is for 'followup-to-me'."
   :type 'boolean
   :group 'wl-draft)
 
+(defcustom wl-draft-delete-myself-from-bcc-fcc nil
+  "*Do not insert bcc or fcc if To and Cc fields is a member of
+`wl-subscribed-mailing-list'"
+  :type 'boolean
+  :group 'wl-draft)
+
 (defcustom wl-draft-resume-folder-window t
   "*Resume folder window in `wl-draft-hide'."
   :type 'boolean
@@ -711,9 +728,12 @@ Default is for 'followup-to-me'."
   :type '(repeat string)
   :group 'wl-summary)
 
-(defcustom wl-summary-fix-timezone "JST"
-  "Non-nil forces to fix timezone of summary date."
-  :type 'string
+(defcustom wl-summary-fix-timezone nil
+  "*Time zone of the date string in summary mode. 
+If nil, it is adjust to the default time zone information
+\(system's default time zone or environment variable TZ\)."
+  :type '(choice (const :tag "Default time zone" nil)
+		 string)
   :group 'wl-summary)
 
 (defcustom wl-summary-default-score 0
@@ -983,15 +1003,24 @@ ex.
   :group 'wl-summary
   :group 'wl-highlight)
 
+(defcustom wl-summary-lazy-highlight (and (boundp 'window-scroll-functions)
+					  (not wl-on-xemacs))
+  "Non-nil forces lazy summary highlighting using `window-scroll-functions'."
+  :type 'boolean
+  :group 'wl-summary
+  :group 'wl-highlight)
+
 (defcustom wl-summary-highlight-partial-threshold 1000
-  "Summary is not highlighted entirely if there are lines more than this value."
+  "Summary is not highlighted entirely if there are lines more than this value.
+Available if only `wl-summary-lazy-highlight' is nil."
   :type 'integer
   :group 'wl-summary
   :group 'wl-highlight)
 
 (defcustom wl-summary-partial-highlight-above-lines 30
   "If Summary has lines more than `wl-summary-highlight-partial-threshold',
-Summary lines are highlighted partialy above current position."
+Summary lines are highlighted partialy above current position.
+Available if only `wl-summary-lazy-highlight' is nil."
   :type 'integer
   :group 'wl-summary
   :group 'wl-highlight)
@@ -1131,6 +1160,16 @@ Each elements are regexp of field-name."
   :group 'wl-summary
   :group 'wl-pref)
 
+(defcustom wl-folder-use-frame nil
+  "*Use dedicated frame for folder mode if non-nil."
+  :type 'boolean
+  :group 'wl-pref)
+
+(defcustom wl-summary-use-frame nil
+  "*Use dedicated frame for each folder summary if non-nil."
+  :type 'boolean
+  :group 'wl-pref)
+
 (defcustom wl-stay-folder-window nil
   "*Stay folder window when folder is selected if non-nil."
   :type 'boolean
@@ -1410,12 +1449,12 @@ every intervals specified by wl-biff-check-interval."
   :type 'integer
   :group 'wl-highlight)
 
-(defcustom wl-biff-state-indicator-on "[$B")(B]"
+(defcustom wl-biff-state-indicator-on "[]"
   "String used to show biff status ON."
   :type 'string
   :group 'wl-highlight)
 
-(defcustom wl-biff-state-indicator-off "[$B!>(B]"
+(defcustom wl-biff-state-indicator-off "[]"
   "String used to show biff status OFF."
   :type 'string
   :group 'wl-highlight)
@@ -2081,20 +2120,17 @@ If it is a number, only numbers will be highlighted."
   :type 'boolean
   :group 'wl-pref)
 
-(defcustom wl-demo-display-logo (or (and (featurep 'xemacs)
-					 (if (featurep 'xpm)
-					     'xpm 'xbm))
-				    (and (module-installed-p 'image)
-					 (if (image-type-available-p 'xpm)
-					     'xpm 'xbm))
-				    (and (module-installed-p 'bitmap)
-					 'xbm))
-  "If non-nil, show graphic logo in the startup screen.
-You can set it to a symbol `xbm' to limit the image format to
-XBM even if XPM can be shown."
-  :type '(radio (const :tag "OFF" nil)
-		(const :tag "XBM (possibly BITMAP-MULE)" xbm)
-		(sexp :format "ON  (any format)" :value t))
+(defcustom wl-demo-display-logo (if (or (featurep 'xemacs)
+					(module-installed-p 'image)
+					(module-installed-p 'bitmap))
+				    t)
+  "If it is T, show graphic logo in the startup screen.  You can set it to
+a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
+  :type '(radio (const :tag "Off" nil)
+		(const :tag "On (any format)" t)
+		(const xpm)
+		(const xbm)
+		(const :tag "bitmap (using BITMAP-MULE)" bitmap))
   :group 'wl-pref)
 
 ;;; Internal variables
@@ -2116,15 +2152,15 @@ XBM even if XPM can be shown."
 ;; Advanced thread view.
 (defvar wl-thread-indent-level 1
   "*Indent level for thread.")
-(defvar wl-thread-have-younger-brother-str "$B(2(B"
+(defvar wl-thread-have-younger-brother-str ""
   "*A string for thread branch line.  It should contain one character.")
-(defvar wl-thread-youngest-child-str       "$B(1(B"
+(defvar wl-thread-youngest-child-str       ""
   "*A string for thread branch line.  It should contain one character.")
-(defvar wl-thread-vertical-str             "$B(-(B"
+(defvar wl-thread-vertical-str             ""
   "*A string for thread branch line.  It should contain one character.")
-(defvar wl-thread-horizontal-str           "$B(,(B"
+(defvar wl-thread-horizontal-str           ""
   "*A string for thread branch line.  It should contain one character.")
-(defvar wl-thread-space-str                "$B!!(B"
+(defvar wl-thread-space-str                ""
   "*A string for thread branch line.  It should contain one character.")
 
 (defvar wl-highlight-thread-indent-string-regexp "[^[<]*"
